/*
   Shader 4 PaperPlanes
*/

#define CAMERABLUR_MAX_SCANFACTOR 7

cbuffer cbChangeOnResize
{
	float2 screen;
};

Texture2D preToon;
Texture2D normal;

cbuffer cbImmutable
{
	float inkThickness = 2.0f;
	float inkBorder = 0.9f;

	float nFilter = 0.8;
	float zFilter = 0.1;

	float nWeight = 1;
	float zWeight = 100;
	
	float gauss[7][15] = {	{0.0019f, 0.0063f, 0.0172f, 0.0394f, 0.0749f, 0.1186f, 0.1562f, 0.1712f, 0.1562f, 0.1186f, 0.0749f, 0.0394f, 0.0172f, 0.0063f, 0.0019f},
							{0.0000f, 0.0022f, 0.0088f, 0.0270f, 0.0648f, 0.1211f, 0.1762f, 0.1997f, 0.1762f, 0.1211f, 0.0648f, 0.0270f, 0.0088f, 0.0022f, 0.0000f},
							{0.0000f, 0.0000f, 0.0027f, 0.0134f, 0.0474f, 0.1166f, 0.2001f, 0.2396f, 0.2001f, 0.1166f, 0.0474f, 0.0134f, 0.0027f, 0.0000f, 0.0000f},
							{0.0000f, 0.0000f, 0.0000f, 0.0033f, 0.0238f, 0.0972f, 0.2260f, 0.2994f, 0.2260f, 0.0972f, 0.0238f, 0.0033f, 0.0000f, 0.0000f, 0.0000f},
							{0.0000f, 0.0000f, 0.0000f, 0.0000f, 0.0044f, 0.0540f, 0.2420f, 0.3991f, 0.2420f, 0.0540f, 0.0044f, 0.0000f, 0.0000f, 0.0000f, 0.0000f},
							{0.0000f, 0.0000f, 0.0000f, 0.0000f, 0.0000f, 0.0066f, 0.1942f, 0.5983f, 0.1942f, 0.0066f, 0.0000f, 0.0000f, 0.0000f, 0.0000f, 0.0000f},
							{0.0000f, 0.0000f, 0.0000f, 0.0000f, 0.0000f, 0.0000f, 0.0109f, 0.9783f, 0.0109f, 0.0000f, 0.0000f, 0.0000f, 0.0000f, 0.0000f, 0.0000f}
	};
};

//4 blur
cbuffer cbChangesEveryFrame
{
	float NoBlurRadius = 0.5f;
	float FullBlurRadius = 0.8f;
	float2 BlurCenterOffset = 0;
};


SamplerState DecalSampler
{
   Filter = MIN_MAG_MIP_LINEAR;
   AddressU = Wrap;
   AddressV = Wrap;
};

DepthStencilState DepthStencilInfo
{
   DepthEnable = true;
   DepthWriteMask = ALL;
   DepthFunc = Less;

   // Setup stencil states
   StencilEnable = true;
   StencilReadMask = 0xFF;
   StencilWriteMask = 0x00;

   FrontFaceStencilFunc = Not_Equal;
   FrontFaceStencilPass = Keep;
   FrontFaceStencilFail = Zero;

   BackFaceStencilFunc = Not_Equal;
   BackFaceStencilPass = Keep;
   BackFaceStencilFail = Zero;
};

struct VS_INPUT
{
   float4 Pos : POSITION;
   float2 Tex : TEXCOORD;
};

struct GS_INPUT
{
   float4 Pos : SV_POSITION;
   float2 Tex : TEXCOORD;
};

struct PS_INPUT
{
   float4 Pos : SV_POSITION;
   float2 Tex : TEXCOORD;
};


GS_INPUT FinalVS(VS_INPUT input)
{
   GS_INPUT output = (GS_INPUT)0;
	output.Pos = input.Pos;
	output.Tex = input.Tex;	

   return output;
}

float calcChangeSum(float2 tex) {
	float2 sampleDirection = (inkThickness/2.0f) / screen;
	float4 o = preToon.Sample(DecalSampler, tex);
	
	/* sample points:
	3-2
	-O-
	1-4
	*/
	float4 s1 = preToon.Sample(DecalSampler, tex + float2(-1, -1) * sampleDirection);
	float4 s2 = preToon.Sample(DecalSampler, tex + float2( 1,  1) * sampleDirection);
	float4 s3 = preToon.Sample(DecalSampler, tex + float2(-1,  1) * sampleDirection);
	float4 s4 = preToon.Sample(DecalSampler, tex + float2( 1, -1) * sampleDirection);
	
	//look if there is another object touching
	float z = abs(s1.w - o.w) + abs(s2.w - o.w) + abs(s3.w - o.w) + abs(s4.w - o.w);
	if(z >= 1)
		return 1;

	// how much are the values changing?
	//eigtl abs(abs(s1 - o) - abs(s2 - o)) + abs(abs(s3 - o) - abs(s4 - o))
	float3 deltaChange = abs(s1 - s2) + abs(s3 - s4);

	//saturate/filter small changes and give it a weight
	return saturate((dot(deltaChange, 1) - nFilter) * nWeight);
}

[maxvertexcount(3)]
void FinalGS(triangle GS_INPUT input[3], inout TriangleStream<PS_INPUT> triStream)
{

	PS_INPUT output = (PS_INPUT)0;
	output.Pos = input[0].Pos;
	output.Tex = input[0].Tex;
	triStream.Append(output);
	
	output.Pos = input[1].Pos;
	output.Tex = input[1].Tex;
	triStream.Append(output);
	
	output.Pos = input[2].Pos;
	output.Tex = input[2].Tex;
	triStream.Append(output);

	triStream.RestartStrip();
}

float4 FinalPS(PS_INPUT input) : SV_Target
{
	float4 origColor = normal.Sample(DecalSampler, input.Tex);

	float sumC = calcChangeSum(input.Tex);
    
    float4 pincelColor = 0;
    pincelColor.w=1;
	
	origColor *= (1-sumC);
	
	return origColor - pincelColor;
}


float4 FinalPS_NoToon(PS_INPUT input) : SV_Target
{
	return normal.Sample(DecalSampler, input.Tex);
}

float4 FinalPSN(PS_INPUT input) : SV_Target
{
	//return abs(preToon.Sample(DecalSampler, input.Tex));
	/*float4 s = preToon.Sample(DecalSampler, input.Tex);
	float4 r = 0;
	r.x=abs(s.r); 
	r.y=abs(s.g);
	r.z=abs(s.b);
	r.w=1;
	return r;/**/
	float s = preToon.Sample(DecalSampler, input.Tex);
	float4 r = 0;
	r.x=s; 
	r.y=s;
	r.z=s;
	r.w=1;
	return r;/**/
}

float4 CameraBlurPS(PS_INPUT input) : SV_Target
{
	float4 r = 0, s=0; int ct = 0;
	int scanFactor = 0;
	
	if((NoBlurRadius < FullBlurRadius) && NoBlurRadius>0 && FullBlurRadius>0) {
		float radius = length(float2(input.Tex.x*2-1, input.Tex.y*-2+1) - BlurCenterOffset);
		if(radius < NoBlurRadius)
			return normal.Sample(DecalSampler, input.Tex);
		if(radius > FullBlurRadius) {
			scanFactor = CAMERABLUR_MAX_SCANFACTOR;
		}
		else {
			scanFactor = floor(CAMERABLUR_MAX_SCANFACTOR * (radius - NoBlurRadius)/(FullBlurRadius - NoBlurRadius));
		}
	}
	
	if(scanFactor==0 || scanFactor>CAMERABLUR_MAX_SCANFACTOR)
		return normal.Sample(DecalSampler, input.Tex);
	
	for(int i= -CAMERABLUR_MAX_SCANFACTOR; i <= CAMERABLUR_MAX_SCANFACTOR; i++) {
		if( (i > -scanFactor) && (i < scanFactor) )
		s=0;
		for(int j= -CAMERABLUR_MAX_SCANFACTOR; j <= CAMERABLUR_MAX_SCANFACTOR; j++) {
			if( (j > -scanFactor) && (j < scanFactor) )
				s += normal.Sample(DecalSampler, input.Tex, int2(i,j)) * gauss[CAMERABLUR_MAX_SCANFACTOR-scanFactor][j+CAMERABLUR_MAX_SCANFACTOR];
		}
		r += s * gauss[CAMERABLUR_MAX_SCANFACTOR-scanFactor][i+CAMERABLUR_MAX_SCANFACTOR];
	}
	return r;
}

float4 NoCameraBlurPS(PS_INPUT input) : SV_Target
{
	return normal.Sample(DecalSampler, input.Tex);
}

technique10 Final
{
   pass P0
   {
      SetDepthStencilState(DepthStencilInfo, 0);
	  

      SetVertexShader(CompileShader(vs_4_0, FinalVS()));
      SetGeometryShader(CompileShader(gs_4_0, FinalGS()));
      SetPixelShader(CompileShader(ps_4_0, FinalPS()));
   }
}


technique10 FinalN
{
   pass P0
   {
      SetDepthStencilState(DepthStencilInfo, 0);
	  

      SetVertexShader(CompileShader(vs_4_0, FinalVS()));
      SetGeometryShader(CompileShader(gs_4_0, FinalGS()));
      SetPixelShader(CompileShader(ps_4_0, FinalPSN()));
   }
}

technique10 CameraBlur
{
   pass P0
   {
      SetDepthStencilState(DepthStencilInfo, 0);
	  

      SetVertexShader(CompileShader(vs_4_0, FinalVS()));
      SetGeometryShader(CompileShader(gs_4_0, FinalGS()));
      SetPixelShader(CompileShader(ps_4_0, CameraBlurPS()));
   }
}

technique10 NoCameraBlur
{
   pass P0
   {
      SetDepthStencilState(DepthStencilInfo, 0);
	  

      SetVertexShader(CompileShader(vs_4_0, FinalVS()));
      SetGeometryShader(CompileShader(gs_4_0, FinalGS()));
      SetPixelShader(CompileShader(ps_4_0, NoCameraBlurPS()));
   }
}