/*
	Shader for Particles
*/
#define CONTRAIL_PARTICLE_TTL 0.2f //!!!!!!!!!sync this value in contrails.hpp (or anywhere else, where the particles are created)


struct VS_INPUT {
	float4 Pos : POSITION;
	float Time : TEXCOORD;
};

struct VS_OUTPUT {
	float4 particlePosition : POSITION;
	float4 particleColor : COLOR;
	float radius : PSIZE;
};

struct PS_INPUT {
	float4 particlePosition : SV_POSITION;
	float4 particleColor : COLOR;
	float2 tex : TEXRURE;
};

Texture2D particleTexture;

SamplerState samplerLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

cbuffer cb0
{
    matrix mWorldViewProj;
    matrix mInvView;
    float3 v3CreationPoint;
    float fGlobalTime;
};

cbuffer cbImmutable
{
    float3 g_positions[4] =
    {
        float3( -1, 1, 0 ),
        float3( 1, 1, 0 ),
        float3( -1, -1, 0 ),
        float3( 1, -1, 0 ),
    };
    float2 g_texcoords[4] = 
    { 
        float2(0,1), 
        float2(1,1),
        float2(0,0),
        float2(1,0),
    };
};

DepthStencilState DepthStencilInfo
{
   DepthEnable = true;
   DepthWriteMask = ALL; //ALL;
   DepthFunc = Less;

   // Setup stencil states
   StencilEnable = true;
   StencilReadMask = 0xFF;
   StencilWriteMask = 0x00;

   FrontFaceStencilFunc = Not_Equal;
   FrontFaceStencilPass = Keep;
   FrontFaceStencilFail = Zero;

   BackFaceStencilFunc = Not_Equal;
   BackFaceStencilPass = Keep;
   BackFaceStencilFail = Zero;
};

DepthStencilState DisableDepth
{
    DepthEnable = FALSE;
    DepthWriteMask = ZERO;
};

DepthStencilState DisableDepthWrite
{
    DepthEnable = TRUE;
    DepthWriteMask = ZERO;
    DepthFunc = ALWAYS;
};

BlendState AdditiveBlending
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;//ONE;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;//0x0F;
};

BlendState AlphaBlending
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};

VS_INPUT VSSceneMain (VS_INPUT input)
{
	return input;
}


[maxvertexcount(4)]
void GSSceneMain(point VS_INPUT input[1], inout TriangleStream<PS_INPUT> outputStream)
{
	PS_INPUT output = (PS_INPUT)0;
	
	float particleAlpha = 1.0f - 
		(fGlobalTime - input[0].Time)/CONTRAIL_PARTICLE_TTL;
	

	//output.particleColor = float4(1.0f, 1.0f, 1.0f, 1.0f
	//	- (totalParticleTime/0.5f));
	//output.particleColor = float4(1.0f, 1.0f, 1.0f, 0.01f);

	output.particleColor = float4(1.0f, 1.0f, 1.0f, particleAlpha * 0.01f);

			
	for(int i = 0; i < 4; i++)
	{
		float3 position = g_positions[i] * 0.01f;
		position = mul(position, (float3x3)mInvView );
		position = position + input[0].Pos;
		
		output.particlePosition = mul( float4(position,1.0), mWorldViewProj );
		output.tex = g_texcoords[i];
		
		outputStream.Append(output);
	}
	
	outputStream.RestartStrip();
}


float4 PSSceneMain(PS_INPUT input) : SV_Target
{
	return particleTexture.Sample(samplerLinear,input.tex) * input.particleColor;

}


technique10 renderParticles
{
   pass P0
   {
      SetVertexShader(CompileShader(vs_4_0, VSSceneMain()));
      SetGeometryShader(CompileShader(gs_4_0, GSSceneMain()));
      SetPixelShader(CompileShader(ps_4_0, PSSceneMain()));
      
      //SetBlendState( AdditiveBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
      SetBlendState( AlphaBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
      SetDepthStencilState( DisableDepthWrite, 0 );
   }
}


//PARTICLE UPDATER

VS_INPUT VSUpdateMain(VS_INPUT input)
{
	return input;
}

[maxvertexcount(10)]
void GSUpdateMain(point VS_INPUT input[1], inout PointStream<VS_INPUT> ParticleOutputStream)
{
	VS_INPUT output = input[0];
	
	float particleTime = fGlobalTime - input[0].Time;
	
	if(particleTime > CONTRAIL_PARTICLE_TTL )
	{
		output.Time = input[0].Time + CONTRAIL_PARTICLE_TTL
			* floor(particleTime/CONTRAIL_PARTICLE_TTL);
		float3 diffVector = input[0].Pos.xyz - v3CreationPoint;
		float vectorBase = (fGlobalTime - output.Time)
			/CONTRAIL_PARTICLE_TTL;
		diffVector *= vectorBase;
		output.Pos.xyz = v3CreationPoint + diffVector;
		output.Pos.a = 1.0f;
	}
	ParticleOutputStream.Append( output );
	ParticleOutputStream.RestartStrip();
}

GeometryShader GSStreamOut = ConstructGSWithSO( CompileShader( gs_4_0, GSUpdateMain() ), "POSITION.xyz; TEXCOORD0.x" );
technique10 updateParticles
{
   pass P0
   {
      SetVertexShader(CompileShader(vs_4_0, VSUpdateMain()));
      SetGeometryShader(GSStreamOut);
      SetPixelShader(NULL);
      
      SetDepthStencilState( DisableDepth, 0 );
   }
}