/*
   Shader 4 PaperPlanes
*/

SamplerState DecalSampler
{
   Filter = MIN_MAG_MIP_LINEAR;
   AddressU = Wrap;
   AddressV = Wrap;
};


DepthStencilState DepthStencilInfo
{
   DepthEnable = true;
   DepthWriteMask = ALL;
   DepthFunc = Less;

   // Setup stencil states
   StencilEnable = true;
   StencilReadMask = 0xFF;
   StencilWriteMask = 0x00;

   FrontFaceStencilFunc = Not_Equal;
   FrontFaceStencilPass = Keep;
   FrontFaceStencilFail = Zero;

   BackFaceStencilFunc = Not_Equal;
   BackFaceStencilPass = Keep;
   BackFaceStencilFail = Zero;
};


cbuffer cbChangesEveryFrame
{
   matrix Model;
   matrix World;
   matrix View;

};

cbuffer cbChangeOnResize
{
   matrix Projection;
};


struct VS_INPUT
{
   float4 Pos : POSITION;
   float3 Norm : NORMAL;
   float2 Tex : TEXCOORD;
};

struct PS_INPUT
{
   float4 Pos : SV_POSITION;
};


PS_INPUT NormalVS(VS_INPUT input)
{
   PS_INPUT output = (PS_INPUT)0;

   float4 Pos = mul(input.Pos, Model);
   Pos = mul(Pos, World);
   Pos = mul(Pos, View);
   output.Pos = mul(Pos, Projection);
   return output;
}

technique10 ShadowMapping
{
   pass P0
   {
      SetDepthStencilState(DepthStencilInfo, 0);

      SetVertexShader(CompileShader(vs_4_0, NormalVS()));
      SetGeometryShader(NULL);
      SetPixelShader(NULL);
   }
}