/*
   Shader 4 PaperPlanes
*/

//VARIABLES
float4 ambientColor;

Texture2D tex;
Texture1D angleMap;
float4 material = 1;


//***enviroment tex samplers****/

SamplerState TexNearestMipOff
{
   Filter = MIN_MAG_MIP_POINT;
   AddressU = Wrap;
   AddressV = Wrap;
   MaxLOD = 0.0f;
};

SamplerState TexNearestMipNearest
{
   Filter = MIN_MAG_MIP_POINT;
   AddressU = Wrap;
   AddressV = Wrap;
};

SamplerState TexNearestMipLinear
{
   Filter = MIN_MAG_POINT_MIP_LINEAR;
   AddressU = Wrap;
   AddressV = Wrap;
};

SamplerState TexBilinearMipOff
{
   Filter = MIN_MAG_LINEAR_MIP_POINT;
   AddressU = Wrap;
   AddressV = Wrap;
   MaxLOD = 0.0f;
};

SamplerState TexBilinearMipNearest
{
   Filter = MIN_MAG_LINEAR_MIP_POINT;
   AddressU = Wrap;
   AddressV = Wrap;
};

SamplerState TexBilinearMipLinear
{
   Filter = MIN_MAG_MIP_LINEAR;
   AddressU = Wrap;
   AddressV = Wrap;
};

//******************************/

SamplerState AngleMapSampler
{
   Filter = MIN_MAG_MIP_LINEAR;
   AddressU = Wrap;
};

DepthStencilState DepthStencilInfo
{
   DepthEnable = true;
   DepthWriteMask = ALL;
   DepthFunc = Less_Equal;

   // Setup stencil states
   StencilEnable = true;
   StencilReadMask = 0xFF;
   StencilWriteMask = 0x00;

   FrontFaceStencilFunc = Not_Equal;
   FrontFaceStencilPass = Keep;
   FrontFaceStencilFail = Zero;

   BackFaceStencilFunc = Not_Equal;
   BackFaceStencilPass = Keep;
   BackFaceStencilFail = Zero;
};


cbuffer cbChangesEveryFrame
{
   float4 wPos;
   float3 cameraPos;
   float2 rect;
   float2 anchor;
   matrix View;
   matrix ViewI;
   //***sampler vars***
   int mipMode = 2;
   bool texBilinear = true;
};

cbuffer cbChangeOnResize
{
   matrix Projection;
   float2 screen;
};


struct VS_INPUT
{
   float4 Pos : POSITION;
   float2 Tex : TEXCOORD;
};

struct PS_INPUT
{
   float4 Pos : SV_POSITION;
   float3 Norm : NORMAL;
   float2 Tex : TEXCOORD;
   float4 worldPos: TEXCOORD1;
   float3 viewV: TEXCOORD2;
};

float4 sampleTex(Texture2D t, float2 uv) {
	if(!texBilinear) {
		switch(mipMode) {
			case 0:
				return t.Sample(TexNearestMipOff,uv);
			case 1:
				return t.Sample(TexNearestMipNearest,uv);
			case 2:
				return t.Sample(TexNearestMipOff,uv);
			default:
				return t.Sample(TexNearestMipLinear,uv);
		}
	}
	else {
		switch(mipMode) {
			case 0:
				return t.Sample(TexBilinearMipOff,uv);
			case 1:
				return t.Sample(TexBilinearMipNearest,uv);
			case 2:
				return t.Sample(TexBilinearMipOff,uv);
			default:
				return t.Sample(TexBilinearMipLinear,uv);
		}
	}
	return t.Sample(TexBilinearMipLinear,uv);
}

float mapAngle(float angle) {
	return angleMap.Sample(AngleMapSampler, angle);
	//return angle;
}

PS_INPUT spInWorldVS(VS_INPUT input)
{
   PS_INPUT output = (PS_INPUT)0;

   float4 Pos = mul(wPos, View);
   Pos.xy += input.Pos*rect/2; //go to the edge
   output.worldPos = mul(Pos, ViewI);
   output.worldPos = output.worldPos/output.worldPos.w;
   output.Pos = mul(Pos, Projection);
   
   output.Tex = input.Tex;
   
   output.Norm = float3(0,0,-1);
   
   output.viewV = cameraPos - output.worldPos;

   return output;
}


float4 spInWorldPS(PS_INPUT input) : SV_Target
{
	float3 normal = normalize(input.Norm);
	float3 view = normalize(input.viewV);
	
	float4 ambient = ambientColor * material.x;

	float4 color = sampleTex(tex,input.Tex);
	float4 outC = 0;
	outC += color*ambient;
	
	//TODO beleuchtung
	float smD;
	float3 light=input.viewV; //TODO
	float4 lightColor = 0;
	//float4 diffuse = saturate(mapAngle(dot(normal,light))) * lightColor * material.y;// * inShadow;
	//outC += color*diffuse;
	
	return outC;
}

PS_INPUT spInHUDVS(VS_INPUT input)
{
   PS_INPUT output = (PS_INPUT)0;

   float2 Pos = wPos.xy/screen + anchor;
   Pos += input.Pos.xy*rect/screen; //go to the edge
   output.Pos.xy = Pos;
   output.Pos.z = 0;
   output.Pos.w = 1;
   /*output.Pos.xy = wPos.xy/screen;
   output.Pos.xy += input.Pos.xy*rect/2/screen;
   output.Pos.z = 0;
   output.Pos.w = 1;*/
   
   output.Tex = input.Tex;
   
   output.Norm = 0;
   
   output.viewV = 0;

   return output;
}

float4 spInHUDPS(PS_INPUT input) : SV_Target
{
	return sampleTex(tex, input.Tex);
}


technique10 SpriteInWorld
{
   pass P0
   {
      SetDepthStencilState(DepthStencilInfo, 0);

      SetVertexShader(CompileShader(vs_4_0, spInWorldVS()));
      SetGeometryShader(NULL);
      SetPixelShader(CompileShader(ps_4_0, spInWorldPS()));
   }
}


technique10 SpriteInHUD
{
   pass P0
   {
      SetDepthStencilState(DepthStencilInfo, 0);

      SetVertexShader(CompileShader(vs_4_0, spInHUDVS()));
      SetGeometryShader(NULL);
      SetPixelShader(CompileShader(ps_4_0, spInHUDPS()));
   }
}