/*
   Shader 4 PaperPlanes
*/

#define MAX_LIGHT_ARRAY_SIZE 2

//CONSTANTS
float DEPTH_SLOPE_BIAS = 0.3f;
//VARIABLES
float4 ambientColor;
float4 specularColor;
Texture2D decal;
Texture1D angleMap;

int sampleDirs[3] = {-1, 0, 1};

int smMapCount;
Texture2D smMap[MAX_LIGHT_ARRAY_SIZE];
float smDistance[MAX_LIGHT_ARRAY_SIZE];
float4 smPos[MAX_LIGHT_ARRAY_SIZE];
float smWidth[MAX_LIGHT_ARRAY_SIZE];
float smHeight[MAX_LIGHT_ARRAY_SIZE];
bool smIsRound[MAX_LIGHT_ARRAY_SIZE];
bool smIsActive[MAX_LIGHT_ARRAY_SIZE];
matrix smView[MAX_LIGHT_ARRAY_SIZE];
matrix smProj[MAX_LIGHT_ARRAY_SIZE];
float4 smLightColor[MAX_LIGHT_ARRAY_SIZE];


//***enviroment tex samplers****/

SamplerState TexNearestMipOff
{
   Filter = MIN_MAG_MIP_POINT;
   AddressU = Wrap;
   AddressV = Wrap;
   MaxLOD = 0.0f;
};

SamplerState TexNearestMipNearest
{
   Filter = MIN_MAG_MIP_POINT;
   AddressU = Wrap;
   AddressV = Wrap;
};

SamplerState TexNearestMipLinear
{
   Filter = MIN_MAG_POINT_MIP_LINEAR;
   AddressU = Wrap;
   AddressV = Wrap;
};

SamplerState TexBilinearMipOff
{
   Filter = MIN_MAG_LINEAR_MIP_POINT;
   AddressU = Wrap;
   AddressV = Wrap;
   MaxLOD = 0.0f;
};

SamplerState TexBilinearMipNearest
{
   Filter = MIN_MAG_LINEAR_MIP_POINT;
   AddressU = Wrap;
   AddressV = Wrap;
};

SamplerState TexBilinearMipLinear
{
   Filter = MIN_MAG_MIP_LINEAR;
   AddressU = Wrap;
   AddressV = Wrap;
};

//******************************/

SamplerState AngleMapSampler
{
   //Filter = MIN_MAG_MIP_LINEAR;
   Filter = MIN_MAG_MIP_POINT;
   AddressU = CLAMP;
};

SamplerComparisonState ShadowSampler
{
   // sampler state
   Filter = COMPARISON_MIN_MAG_LINEAR_MIP_POINT;
  AddressU = CLAMP;
  AddressV = CLAMP;

   // sampler comparison state
   ComparisonFunc = LESS_EQUAL; 
};


DepthStencilState DepthStencilInfo
{
   DepthEnable = true;
   DepthWriteMask = ALL;
   DepthFunc = Less;

   // Setup stencil states
   StencilEnable = true;
   StencilReadMask = 0xFF;
   StencilWriteMask = 0x00;

   FrontFaceStencilFunc = Not_Equal;
   FrontFaceStencilPass = Keep;
   FrontFaceStencilFail = Zero;

   BackFaceStencilFunc = Not_Equal;
   BackFaceStencilPass = Keep;
   BackFaceStencilFail = Zero;
};

BlendState ToonNormalBlending
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = TRUE;
    BlendEnable[1] = FALSE;
    /*SrcBlend = BLEND_ONE;
	DestBlend = BLEND_ZERO;
	BlendOp	= BLEND_OP_ADD;
	SrcBlendAlpha = BLEND_ONE;
	DestBlendAlpha = BLEND_ZERO;
	BlendOpAlpha = BLEND_OP_ADD;*/
    
    //SrcBlend = SRC_ALPHA;
    //DestBlend = SRC_ALPHA;
    //BlendOp = ADD;
    //SrcBlendAlpha = SRC_ALPHA;
    //DestBlendAlpha = SRC_ALPHA;
    //BlendOpAlpha = ADD;
    //RenderTargetWriteMask[0] = 0x0F;
};

cbuffer cbChangesEveryFrame
{
   matrix Model;
   matrix World;
   matrix View;
   float4 surface;
   float3 cameraPos;
   int mipMode = 2; //***sampler var***
   int objNumber; 
   bool texBilinear = true; //***sampler var***
};

cbuffer cbChangeOnResize
{
   matrix Projection;
};


struct VS_INPUT
{
   float4 Pos : POSITION;
   float3 Norm : NORMAL;
   float2 Tex : TEXCOORD;
};

struct GS_INPUT
{
   float4 Pos : SV_POSITION;
   float3 Norm : NORMAL;
   float2 Tex : TEXCOORD;
   float4 worldPos: TEXCOORD1;
   float3 viewV: TEXCOORD2;
};

struct PS_INPUT
{
   float4 Pos : SV_POSITION;
   float3 Norm : NORMAL;
   float2 Tex : TEXCOORD;
   float4 worldPos: TEXCOORD1;
   float3 viewV: TEXCOORD2;
};

struct ShaderOut
{
	float4 color : SV_TARGET0;
    float4 normz : SV_TARGET1;
};

float mapAngle(float angle) {
	if(angle <= 0.0f)
		return 0.0f;
	if(angle >= 1.0f)
		return 1.0f;
	return angleMap.Sample(AngleMapSampler, angle);
}

float4 sampleTex(Texture2D t, float2 uv) {
	if(!texBilinear) {
		switch(mipMode) {
			case 0:
				return t.Sample(TexNearestMipOff,uv);
			case 1:
				return t.Sample(TexNearestMipNearest,uv);
			case 2:
				return t.Sample(TexNearestMipLinear,uv);
			default:
				return t.Sample(TexNearestMipLinear,uv);
		}
	}
	else {
		switch(mipMode) {
			case 0:
				return t.Sample(TexBilinearMipOff,uv);
			case 1:
				return t.Sample(TexBilinearMipNearest,uv);
			case 2:
				return t.Sample(TexBilinearMipLinear,uv);
			default:
				return t.Sample(TexBilinearMipLinear,uv);
		}
	}
	return t.Sample(TexBilinearMipLinear,uv);
}

PS_INPUT NormalVS(VS_INPUT input)
{
   PS_INPUT output = (PS_INPUT)0;

   float4 Pos = mul(input.Pos, Model);
   Pos = mul(Pos, World);
   output.worldPos = Pos/Pos.w;
   Pos = mul(Pos, View);
   output.Pos = mul(Pos, Projection);
   
   output.Tex = input.Tex;
   
   output.Norm = mul(input.Norm, (float3x3)Model);
   output.Norm = mul(output.Norm, (float3x3)World);
   //output.Norm = mul(output.Norm, (float3x3)View);
   //output.Norm = mul(output.Norm, (float3x3)Projection);
   
   output.viewV = cameraPos - output.worldPos;

   return output;
}

[maxvertexcount(3)]
void NormalGS(triangle GS_INPUT input[3], inout TriangleStream<PS_INPUT> triStream)
{
	PS_INPUT output = (PS_INPUT)0;
	/*? visible calc???*/
	output.Pos = input[0].Pos;
	output.Norm = input[0].Norm;
	output.worldPos = input[0].worldPos;
	output.viewV = input[0].viewV;
	output.Tex = input[0].Tex;
	triStream.Append(output);
	
	output.Pos = input[1].Pos;
	output.Norm = input[1].Norm;
	output.worldPos = input[1].worldPos;
	output.viewV = input[1].viewV;
	output.Tex = input[1].Tex;
	triStream.Append(output);
	
	output.Pos = input[2].Pos;
	output.Norm = input[2].Norm;
	output.worldPos = input[2].worldPos;
	output.viewV = input[2].viewV;
	output.Tex = input[2].Tex;
	triStream.Append(output);

	triStream.RestartStrip();
}


ShaderOut NormalPS(PS_INPUT input) : SV_Target
{
	ShaderOut so = (ShaderOut) 0;
	/* COLOR CALC */
	float3 normal = normalize(input.Norm);
	float3 view = normalize(input.viewV);
	
	if(dot(view,normal)<0)
		normal *= -1;
	
	float4 ambient = ambientColor * surface.x;

	float4 color = sampleTex(decal,input.Tex);
	float origAplha = color.w;

	so.color = color*ambient;
	int i;
	for(i=0; i < MAX_LIGHT_ARRAY_SIZE; ++i) {
		if(i < smMapCount) {
			if(smIsActive[i]) {
				float4 lightPos = smPos[i];
				float l = length(lightPos-input.worldPos);
				float d = smDistance[i];
				if(l<d) {
					float3 light = normalize( lightPos.xyz - input.worldPos.xyz );
					float3 halfV = normalize(light + view);
					
					float4 smkoord = mul(mul(input.worldPos, smView[i]), smProj[i]);
					smkoord = smkoord/smkoord.w;
					if(abs(smkoord.x)<=1 && abs(smkoord.y)<=1 && smkoord.z<=1 && smkoord.z>0) {//matched
					//if(abs(smkoord.x)<=1 && abs(smkoord.y)<=1 && abs(smkoord.z)<=1) {//matched
						if((smIsRound[i]==false) || (length(smkoord.xy)<=1) ) {
							float2 smtex;
							smtex.x = (smkoord.x+1.0f)/2.0f;
							smtex.y = (smkoord.y-1.0f)/-2.0f;
							
							float inShadow = 0;
							float2 smSize=0;
							smSize.x = 1.0f/smWidth[i];
							smSize.y = 1.0f/smHeight[i];
							
							
							//my own pcf
							/*	float smD;
							smD = smMap[i].Sample(DecalSampler,  smtex + float2(-1,-1)*smSize);	smD += DEPTH_BIAS;	if(smkoord.z<=smD) inShadow++;
							smD = smMap[i].Sample(DecalSampler,  smtex + float2(-1, 0)*smSize);	smD += DEPTH_BIAS;	if(smkoord.z<=smD) inShadow++;
							smD = smMap[i].Sample(DecalSampler,  smtex + float2(-1, 1)*smSize);	smD += DEPTH_BIAS;	if(smkoord.z<=smD) inShadow++;
							smD = smMap[i].Sample(DecalSampler,  smtex + float2( 0,-1)*smSize);	smD += DEPTH_BIAS;	if(smkoord.z<=smD) inShadow++;
							smD = smMap[i].Sample(DecalSampler,  smtex + float2( 0, 0)*smSize);	smD += DEPTH_BIAS;	if(smkoord.z<=smD) inShadow++;
							smD = smMap[i].Sample(DecalSampler,  smtex + float2( 0, 1)*smSize);	smD += DEPTH_BIAS;	if(smkoord.z<=smD) inShadow++;
							smD = smMap[i].Sample(DecalSampler,  smtex + float2( 1,-1)*smSize);	smD += DEPTH_BIAS;	if(smkoord.z<=smD) inShadow++;
							smD = smMap[i].Sample(DecalSampler,  smtex + float2( 1, 0)*smSize);	smD += DEPTH_BIAS;	if(smkoord.z<=smD) inShadow++;
							smD = smMap[i].Sample(DecalSampler,  smtex + float2( 1, 1)*smSize);	smD += DEPTH_BIAS;	if(smkoord.z<=smD) inShadow++;

							inShadow = inShadow/9;/**/
							
							//one pcf step
							/*smkoord.z -= DEPTH_BIAS;
							inShadow = smMap[i].SampleCmpLevelZero( ShadowSampler, smtex, smkoord.z);/**/
							
							//several pcf steps
							//float slopeBias = (1-saturate(dot(normal,float3(0,0,-1))))*DEPTH_SLOPE_BIAS;
							//smkoord.z -= 1/d + slopeBias; //1/d = standard depth bias (d=distance of shadowmap)
							inShadow += smMap[i].SampleCmpLevelZero(ShadowSampler,  smtex + float2(-1,-1)*smSize, smkoord.z);
							inShadow += smMap[i].SampleCmpLevelZero(ShadowSampler,  smtex + float2(-1, 0)*smSize, smkoord.z);
							inShadow += smMap[i].SampleCmpLevelZero(ShadowSampler,  smtex + float2(-1, 1)*smSize, smkoord.z);
							inShadow += smMap[i].SampleCmpLevelZero(ShadowSampler,  smtex + float2( 0,-1)*smSize, smkoord.z);
							inShadow += smMap[i].SampleCmpLevelZero(ShadowSampler,  smtex + float2( 0, 0)*smSize, smkoord.z);	
							inShadow += smMap[i].SampleCmpLevelZero(ShadowSampler,  smtex + float2( 0, 1)*smSize, smkoord.z);
							inShadow += smMap[i].SampleCmpLevelZero(ShadowSampler,  smtex + float2( 1,-1)*smSize, smkoord.z);
							inShadow += smMap[i].SampleCmpLevelZero(ShadowSampler,  smtex + float2( 1, 0)*smSize, smkoord.z);
							inShadow += smMap[i].SampleCmpLevelZero(ShadowSampler,  smtex + float2( 1, 1)*smSize, smkoord.z);	
							inShadow = inShadow/9;/**/
							
							float4 diffuse = mapAngle(saturate(dot(normal,light))) * smLightColor[i] * surface.y * inShadow;
							//float4 specular = pow(mapAngle(saturate(dot(normal, halfV))), surface.w) * specularColor * surface.z * inShadow;
							//surface.z *= -1;
							float4 specular = 0.0f;
							so.color += color * (diffuse + specular);
						}
					}
				}
			}
		}
	}
	so.color.w = origAplha;

	/* NORMAL CALC */
	so.normz.xyz = normal.xyz;
	so.normz.w = objNumber;

	return so;
}

technique10 ColorNormZ
{
   pass P0
   {
	  //SetBlendState( ToonNormalBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
      SetDepthStencilState(DepthStencilInfo, 0);

      SetVertexShader(CompileShader(vs_4_0, NormalVS()));
      //SetGeometryShader(CompileShader(gs_4_0, NormalGS()));
      SetGeometryShader(NULL);
      SetPixelShader(CompileShader(ps_4_0, NormalPS()));
   }
}